import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.StringTokenizer;

/**
 * Esta clase fornece ajuda para obter informao sobre o estado da rede num computador com linux.
 * 
 */
public class Util {
//	private static ArrayList<String> _netifs;

	//associamos cada estado a um numero, se o sistema tiver configurado para outro idioma deve traduzir-se
	static HashMap<String, Integer> stateMap = new HashMap<String, Integer>();
	static {
		stateMap.put("LISTEN", 2);

		stateMap.put("SYN_SENT", 3);
		stateMap.put("SYN_RECV", 4);
		stateMap.put("ESTABLISHED", 5);

		stateMap.put("FIN_WAIT1", 6);
		stateMap.put("FIN_WAIT_1", 6);
		stateMap.put("FIN_WAIT2", 7);

		stateMap.put("FIN_WAIT_2", 7);
		stateMap.put("CLOSE_WAIT", 8);
		stateMap.put("LAST_ACK", 9);

		stateMap.put("CLOSING", 10);
		stateMap.put("TIME_WAIT", 11);
		stateMap.put("DELETE_TCB", 12);

	}

	/**
	 * Separa um String em vrios String delimitados por <code>delim</code> e devolve um arranjo
	 * com esses elementos.
	 */
	public static String[] tokenize(String text, String delim) {
		StringTokenizer st = new StringTokenizer(text, delim);

		int size = st.countTokens();
		String[] ret = new String[size];
		int i = 0;
		while (st.hasMoreTokens()) {
			ret[i++] = st.nextToken();
		}
		return ret;

	}

	/**
	 * Executa o comando netstat e devolve uma lista de Connections
	 * S pega as conexes TCP
	 */
	public synchronized static ArrayList<Connection> netstat() {
		try

		{
			String cmd = "netstat -p TCP -n";
			Process proc = Runtime.getRuntime().exec(cmd);
			BufferedReader br = new BufferedReader(new InputStreamReader(proc
					.getInputStream()));
			String s;
			String srcIp, destIp, state;

			int srcPort, destPort, stateCode;
			ArrayList<Connection> conns = new ArrayList<Connection>();
			while ((s = br.readLine()) != null)

			{
				s = s.trim();
				//System.err.println(s);
				if (s.startsWith("tcp")) {

					String[] sections = tokenize(s, " ");
					if (sections.length < 4) {
						continue;
					}

					String[] fields = tokenize(sections[3], ":");

					srcIp = fields[0];
					srcPort = Integer.parseInt(fields[1]);
					fields = tokenize(sections[4], ":");

					destIp = fields[0];
					destPort = Integer.parseInt(fields[1]);
					state = sections[5];

					if (state.startsWith("ESTABLISHED"))
						state = "ESTABLISHED";

					try {
						conns.add(new Connection(srcIp, srcPort, destIp,
								destPort, stateMap.get(state)));
					} catch (Exception e) {
						System.out.println(state);
						e.printStackTrace();
					}
				}

			}
			return conns;
		} catch (IOException e) {
			System.out.println(e);
		}
		return null;
	}
//
//	/**
//	 * Suporte para ipconfing
//	 * TODO implementar corretamente para Windows
//	 */
//	public static ArrayList ipconfig()
//
//	{
//		if (_netifs != null) {
//			return _netifs;
//		}
//
//		try {
//			String cmd = "ipconfig /all";
//			Process proc = Runtime.getRuntime().exec(cmd);
//			BufferedReader br = new BufferedReader(new InputStreamReader(proc
//					.getInputStream()));
//			String s;
//
//			String srcIp, destIp, state;
//			int srcPort, destPort, stateCode;
//			ArrayList netifs = new ArrayList();
//			NetInterface netif = new NetInterface();
//
//			netifs.add(netif);
//			netif.ipAddress = "0.0.0.0";
//			netif.descr = "MS TCP Loopback interface";
//
//			int num = 0;
//			while ((s = br.readLine()) != null) {
//				num++;
//
//				s = s.trim();
//				if (s.endsWith(":") && !s.endsWith(". :")) {
//
//					netif = new NetInterface();
//					netifs.add(netif);
//					continue;
//				}
//
//				int pos = s.indexOf(".");
//				if (pos < 0)
//					continue;
//				String name = s.substring(0, pos).trim();
//
//				pos = s.indexOf(":", pos);
//				String value = s.substring(pos + 1);
//				if (name.equals("Description"))
//
//				{
//					netif.descr = value;
//				} else if (name.equals("Subnet Mask"))
//
//				{
//					netif.mask = value;
//				} else if (name.equals("Physical Address"))
//
//				{
//					// netif.physAddress = SnmpOctetString
//					// .convertPhysAddress(value);
//				}
//
//				else if (name.equals("IP Address")) {
//					netif.ipAddress = value;
//				}
//
//			}
//
//			_netifs = netifs;
//			return netifs;
//		}
//
//		catch (IOException e) {
//			System.out.println(e);
//		}
//		return null;
//	}
//
	/**
	 * Testando...
	 */
	public static void main(String... args) {
		// OperatingSystemMXBean o = null;
		// o.getAvailableProcessors ();

		Runtime r = Runtime.getRuntime();
		int numProcessadores = r.availableProcessors();
		System.out.println(numProcessadores);
		ArrayList<Connection> resp = netstat();
		System.out.println(resp.size());
		for (Connection c : resp) {
			System.out.println(c.toString());
		}
	}

}

/**
 * Representa uma conexo do netstat.
 *
 */
class Connection {
	public Connection(String srcIp, int srcPort, String destIp, int destPort,
			int state) {
		this.srcIp = srcIp; // IP de origem
		this.srcPort = srcPort; // Porto de origem
		this.destIp = destIp; // IP destino
		this.destPort = destPort; // Porto destino
		this.state = state; // Estado da conexo

	}

	String srcIp;

	int srcPort;

	String destIp;

	int destPort;

	int state;

	public String toString() {
		return "{srcIp=" + srcIp + ", srcPort=" + srcPort + ", destIp="
				+ destIp + ", destPort=" + destPort + ", state=" + state + "}";
	}
}
